@@ -23,17 +23,27 @@ from distutils.command.sdist import sdist as DistutilsSdist
 
 from setuptools import find_packages as _find_packages
 
-from .distutils_helpers import (add_command_option, get_compiler_option,
-                                get_dummy_distribution, get_distutils_build_option,
-                                get_distutils_build_or_install_option)
+from .distutils_helpers import (
+    add_command_option,
+    get_compiler_option,
+    get_dummy_distribution,
+    get_distutils_build_option,
+    get_distutils_build_or_install_option,
+)
 from .version_helpers import get_pkg_version_module
-from .utils import (walk_skip_hidden, import_file, extends_doc,
-                    resolve_name, AstropyDeprecationWarning)
+from .utils import (
+    walk_skip_hidden,
+    import_file,
+    extends_doc,
+    resolve_name,
+    AstropyDeprecationWarning,
+)
 
 from .commands.build_ext import generate_build_ext_command
 from .commands.build_py import AstropyBuildPy
 from .commands.install import AstropyInstall
 from .commands.install_lib import AstropyInstallLib
+from .commands.register import AstropyRegister
 from .commands.test import AstropyTest
 
 # These imports are not used in this module, but are included for backwards
@@ -41,24 +51,30 @@ from .commands.test import AstropyTest
 from .utils import get_numpy_include_path, write_if_different  # noqa
 from .commands.build_ext import should_build_with_cython, get_compiler_version  # noqa
 
-_module_state = {'registered_commands': None,
-                 'have_sphinx': False,
-                 'package_cache': None}
+_module_state = {
+    "registered_commands": None,
+    "have_sphinx": False,
+    "package_cache": None,
+    "exclude_packages": set(),
+    "excludes_too_late": False,
+}
 
 try:
     import sphinx  # noqa
-    _module_state['have_sphinx'] = True
+
+    _module_state["have_sphinx"] = True
 except ValueError as e:
     # This can occur deep in the bowels of Sphinx's imports by way of docutils
     # and an occurrence of this bug: http://bugs.python.org/issue18378
     # In this case sphinx is effectively unusable
-    if 'unknown locale' in e.args[0]:
+    if "unknown locale" in e.args[0]:
         log.warn(
             "Possible misconfiguration of one of the environment variables "
             "LC_ALL, LC_CTYPES, LANG, or LANGUAGE.  For an example of how to "
             "configure your system's language environment on OSX see "
             "http://blog.remibergsma.com/2012/07/10/"
-            "setting-locales-correctly-on-mac-osx-terminal-application/")
+            "setting-locales-correctly-on-mac-osx-terminal-application/"
+        )
 except ImportError:
     pass
 except SyntaxError:
@@ -93,10 +109,12 @@ def adjust_compiler(package):
     """
 
     warnings.warn(
-        'Direct use of the adjust_compiler function in setup.py is '
-        'deprecated and can be removed from your setup.py.  This '
-        'functionality is now incorporated directly into the build_ext '
-        'command.', AstropyDeprecationWarning)
+        "Direct use of the adjust_compiler function in setup.py is "
+        "deprecated and can be removed from your setup.py.  This "
+        "functionality is now incorporated directly into the build_ext "
+        "command.",
+        AstropyDeprecationWarning,
+    )
 
 
 def get_debug_option(packagename):
@@ -111,63 +129,68 @@ def get_debug_option(packagename):
     """
 
     try:
-        current_debug = get_pkg_version_module(packagename,
-                                               fromlist=['debug'])[0]
+        current_debug = get_pkg_version_module(packagename, fromlist=["debug"])[0]
     except (ImportError, AttributeError):
         current_debug = None
 
     # Only modify the debug flag if one of the build commands was explicitly
     # run (i.e. not as a sub-command of something else)
     dist = get_dummy_distribution()
-    if any(cmd in dist.commands for cmd in ['build', 'build_ext']):
-        debug = bool(get_distutils_build_option('debug'))
+    if any(cmd in dist.commands for cmd in ["build", "build_ext"]):
+        debug = bool(get_distutils_build_option("debug"))
     else:
         debug = bool(current_debug)
 
     if current_debug is not None and current_debug != debug:
-        build_ext_cmd = dist.get_command_class('build_ext')
+        build_ext_cmd = dist.get_command_class("build_ext")
         build_ext_cmd.force_rebuild = True
 
     return debug
 
 
-def register_commands(package, version, release, srcdir='.'):
+def add_exclude_packages(excludes):
 
-    if _module_state['registered_commands'] is not None:
-        return _module_state['registered_commands']
+    if _module_state["excludes_too_late"]:
+        raise RuntimeError(
+            "add_package_excludes must be called before all other setup helper "
+            "functions in order to properly handle excluded packages"
+        )
 
-    if _module_state['have_sphinx']:
+    _module_state["exclude_packages"].update(set(excludes))
+
+
+def register_commands(package, version, release, srcdir="."):
+
+    if _module_state["registered_commands"] is not None:
+        return _module_state["registered_commands"]
+
+    if _module_state["have_sphinx"]:
         try:
-            from .commands.build_sphinx import (AstropyBuildSphinx,
-                                                AstropyBuildDocs)
+            from .commands.build_sphinx import AstropyBuildSphinx, AstropyBuildDocs
         except ImportError:
             AstropyBuildSphinx = AstropyBuildDocs = FakeBuildSphinx
     else:
         AstropyBuildSphinx = AstropyBuildDocs = FakeBuildSphinx
 
-    _module_state['registered_commands'] = registered_commands = {
-        'test': generate_test_command(package),
-
+    _module_state["registered_commands"] = registered_commands = {
+        "test": generate_test_command(package),
         # Use distutils' sdist because it respects package_data.
         # setuptools/distributes sdist requires duplication of information in
         # MANIFEST.in
-        'sdist': DistutilsSdist,
-
+        "sdist": DistutilsSdist,
         # The exact form of the build_ext command depends on whether or not
         # we're building a release version
-        'build_ext': generate_build_ext_command(package, release),
-
+        "build_ext": generate_build_ext_command(package, release),
         # We have a custom build_py to generate the default configuration file
-        'build_py': AstropyBuildPy,
-
+        "build_py": AstropyBuildPy,
         # Since install can (in some circumstances) be run without
         # first building, we also need to override install and
         # install_lib.  See #2223
-        'install': AstropyInstall,
-        'install_lib': AstropyInstallLib,
-
-        'build_sphinx': AstropyBuildSphinx,
-        'build_docs': AstropyBuildDocs
+        "install": AstropyInstall,
+        "install_lib": AstropyInstallLib,
+        "register": AstropyRegister,
+        "build_sphinx": AstropyBuildSphinx,
+        "build_docs": AstropyBuildDocs,
     }
 
     # Need to override the __name__ here so that the commandline options are
@@ -181,17 +204,17 @@ def register_commands(package, version, release, srcdir='.'):
     # Add a few custom options; more of these can be added by specific packages
     # later
     for option in [
-            ('use-system-libraries',
-             "Use system libraries whenever possible", True)]:
-        add_command_option('build', *option)
-        add_command_option('install', *option)
+        ("use-system-libraries", "Use system libraries whenever possible", True)
+    ]:
+        add_command_option("build", *option)
+        add_command_option("install", *option)
 
     add_command_hooks(registered_commands, srcdir=srcdir)
 
     return registered_commands
 
 
-def add_command_hooks(commands, srcdir='.'):
+def add_command_hooks(commands, srcdir="."):
     """
     Look through setup_package.py modules for functions with names like
     ``pre_<command_name>_hook`` and ``post_<command_name>_hook`` where
@@ -203,13 +226,13 @@ def add_command_hooks(commands, srcdir='.'):
     hooks for them (e.g. `AstropyBuildPy`).
     """
 
-    hook_re = re.compile(r'^(pre|post)_(.+)_hook$')
+    hook_re = re.compile(r"^(pre|post)_(.+)_hook$")
 
     # Distutils commands have a method of the same name, but it is not a
     # *classmethod* (which probably didn't exist when distutils was first
     # written)
     def get_command_name(cmdcls):
-        if hasattr(cmdcls, 'command_name'):
+        if hasattr(cmdcls, "command_name"):
             return cmdcls.command_name
         else:
             return cmdcls.__name__
@@ -235,7 +258,8 @@ def add_command_hooks(commands, srcdir='.'):
 
     for cmd_name, cmd_hooks in hooks.items():
         commands[cmd_name] = generate_hooked_command(
-            cmd_name, dist.get_command_class(cmd_name), cmd_hooks)
+            cmd_name, dist.get_command_class(cmd_name), cmd_hooks
+        )
 
 
 def generate_hooked_command(cmd_name, cmd_cls, hooks):
@@ -246,14 +270,20 @@ def generate_hooked_command(cmd_name, cmd_cls, hooks):
     """
 
     def run(self, orig_run=cmd_cls.run):
-        self.run_command_hooks('pre_hooks')
+        self.run_command_hooks("pre_hooks")
         orig_run(self)
-        self.run_command_hooks('post_hooks')
+        self.run_command_hooks("post_hooks")
 
-    return type(cmd_name, (cmd_cls, object),
-                {'run': run, 'run_command_hooks': run_command_hooks,
-                 'pre_hooks': hooks.get('pre', []),
-                 'post_hooks': hooks.get('post', [])})
+    return type(
+        cmd_name,
+        (cmd_cls, object),
+        {
+            "run": run,
+            "run_command_hooks": run_command_hooks,
+            "pre_hooks": hooks.get("pre", []),
+            "post_hooks": hooks.get("post", []),
+        },
+    )
 
 
 def run_command_hooks(cmd_obj, hook_kind):
@@ -274,21 +304,28 @@ def run_command_hooks(cmd_obj, hook_kind):
                 hook_obj = resolve_name(hook)
             except ImportError as exc:
                 raise DistutilsModuleError(
-                    'cannot find hook {0}: {1}'.format(hook, exc))
+                    "cannot find hook {0}: {1}".format(hook, exc)
+                )
         else:
             hook_obj = hook
 
         if not callable(hook_obj):
-            raise DistutilsOptionError('hook {0!r} is not callable' % hook)
+            raise DistutilsOptionError("hook {0!r} is not callable" % hook)
 
-        log.info('running {0} from {1} for {2} command'.format(
-                 hook_kind.rstrip('s'), modname, cmd_obj.get_command_name()))
+        log.info(
+            "running {0} from {1} for {2} command".format(
+                hook_kind.rstrip("s"), modname, cmd_obj.get_command_name()
+            )
+        )
 
         try:
             hook_obj(cmd_obj)
         except Exception:
-            log.error('{0} command hook {1} raised an exception: %s\n'.format(
-                hook_obj.__name__, cmd_obj.get_command_name()))
+            log.error(
+                "{0} command hook {1} raised an exception: %s\n".format(
+                    hook_obj.__name__, cmd_obj.get_command_name()
+                )
+            )
             log.error(traceback.format_exc())
             sys.exit(1)
 
@@ -300,12 +337,12 @@ def generate_test_command(package_name):
     tested.
     """
 
-    return type(package_name.title() + 'Test', (AstropyTest,),
-                {'package_name': package_name})
+    return type(
+        package_name.title() + "Test", (AstropyTest,), {"package_name": package_name}
+    )
 
 
-def update_package_files(srcdir, extensions, package_data, packagenames,
-                         package_dirs):
+def update_package_files(srcdir, extensions, package_data, packagenames, package_dirs):
     """
     This function is deprecated and maintained for backward compatibility
     with affiliated packages.  Affiliated packages should update their
@@ -313,13 +350,13 @@ def update_package_files(srcdir, extensions, package_data, packagenames,
     """
 
     info = get_package_info(srcdir)
-    extensions.extend(info['ext_modules'])
-    package_data.update(info['package_data'])
-    packagenames = list(set(packagenames + info['packages']))
-    package_dirs.update(info['package_dir'])
+    extensions.extend(info["ext_modules"])
+    package_data.update(info["package_data"])
+    packagenames = list(set(packagenames + info["packages"]))
+    package_dirs.update(info["package_dir"])
 
 
-def get_package_info(srcdir='.', exclude=()):
+def get_package_info(srcdir=".", exclude=()):
     """
     Collates all of the information for building all subpackages
     and returns a dictionary of keyword arguments that can
@@ -368,68 +405,74 @@ def get_package_info(srcdir='.', exclude=()):
     package_dir = {}
     skip_2to3 = []
 
+    if exclude:
+        warnings.warn(
+            "Use of the exclude parameter is no longer supported since it does "
+            "not work as expected. Use add_exclude_packages instead. Note that "
+            "it must be called prior to any other calls from setup helpers.",
+            AstropyDeprecationWarning,
+        )
+
     # Use the find_packages tool to locate all packages and modules
     packages = filter_packages(find_packages(srcdir, exclude=exclude))
 
     # Update package_dir if the package lies in a subdirectory
-    if srcdir != '.':
-        package_dir[''] = srcdir
+    if srcdir != ".":
+        package_dir[""] = srcdir
 
     # For each of the setup_package.py modules, extract any
     # information that is needed to install them.  The build options
     # are extracted first, so that their values will be available in
     # subsequent calls to `get_extensions`, etc.
     for setuppkg in iter_setup_packages(srcdir, packages):
-        if hasattr(setuppkg, 'get_build_options'):
+        if hasattr(setuppkg, "get_build_options"):
             options = setuppkg.get_build_options()
             for option in options:
-                add_command_option('build', *option)
-        if hasattr(setuppkg, 'get_external_libraries'):
+                add_command_option("build", *option)
+        if hasattr(setuppkg, "get_external_libraries"):
             libraries = setuppkg.get_external_libraries()
             for library in libraries:
                 add_external_library(library)
-        if hasattr(setuppkg, 'requires_2to3'):
+        if hasattr(setuppkg, "requires_2to3"):
             requires_2to3 = setuppkg.requires_2to3()
         else:
             requires_2to3 = True
         if not requires_2to3:
-            skip_2to3.append(
-                os.path.dirname(setuppkg.__file__))
+            skip_2to3.append(os.path.dirname(setuppkg.__file__))
 
     for setuppkg in iter_setup_packages(srcdir, packages):
         # get_extensions must include any Cython extensions by their .pyx
         # filename.
-        if hasattr(setuppkg, 'get_extensions'):
+        if hasattr(setuppkg, "get_extensions"):
             ext_modules.extend(setuppkg.get_extensions())
-        if hasattr(setuppkg, 'get_package_data'):
+        if hasattr(setuppkg, "get_package_data"):
             package_data.update(setuppkg.get_package_data())
 
     # Locate any .pyx files not already specified, and add their extensions in.
     # The default include dirs include numpy to facilitate numerical work.
-    ext_modules.extend(get_cython_extensions(srcdir, packages, ext_modules,
-                                             ['numpy']))
+    ext_modules.extend(get_cython_extensions(srcdir, packages, ext_modules, ["numpy"]))
 
     # Now remove extensions that have the special name 'skip_cython', as they
     # exist Only to indicate that the cython extensions shouldn't be built
     for i, ext in reversed(list(enumerate(ext_modules))):
-        if ext.name == 'skip_cython':
+        if ext.name == "skip_cython":
             del ext_modules[i]
 
     # On Microsoft compilers, we need to pass the '/MANIFEST'
     # commandline argument.  This was the default on MSVC 9.0, but is
     # now required on MSVC 10.0, but it doesn't seem to hurt to add
     # it unconditionally.
-    if get_compiler_option() == 'msvc':
+    if get_compiler_option() == "msvc":
         for ext in ext_modules:
-            ext.extra_link_args.append('/MANIFEST')
+            ext.extra_link_args.append("/MANIFEST")
 
     return {
-        'ext_modules': ext_modules,
-        'packages': packages,
-        'package_dir': package_dir,
-        'package_data': package_data,
-        'skip_2to3': skip_2to3
-        }
+        "ext_modules": ext_modules,
+        "packages": packages,
+        "package_dir": package_dir,
+        "package_data": package_data,
+        "skip_2to3": skip_2to3,
+    }
 
 
 def iter_setup_packages(srcdir, packages):
@@ -445,14 +488,12 @@ def iter_setup_packages(srcdir, packages):
     """
 
     for packagename in packages:
-        package_parts = packagename.split('.')
+        package_parts = packagename.split(".")
         package_path = os.path.join(srcdir, *package_parts)
-        setup_package = os.path.relpath(
-            os.path.join(package_path, 'setup_package.py'))
+        setup_package = os.path.relpath(os.path.join(package_path, "setup_package.py"))
 
         if os.path.isfile(setup_package):
-            module = import_file(setup_package,
-                                 name=packagename + '.setup_package')
+            module = import_file(setup_package, name=packagename + ".setup_package")
             yield module
 
 
@@ -471,17 +512,18 @@ def iter_pyx_files(package_dir, package_name):
     """
     for dirpath, dirnames, filenames in walk_skip_hidden(package_dir):
         for fn in filenames:
-            if fn.endswith('.pyx'):
+            if fn.endswith(".pyx"):
                 fullfn = os.path.relpath(os.path.join(dirpath, fn))
                 # Package must match file name
-                extmod = '.'.join([package_name, fn[:-4]])
+                extmod = ".".join([package_name, fn[:-4]])
                 yield (extmod, fullfn)
 
         break  # Don't recurse into subdirectories
 
 
-def get_cython_extensions(srcdir, packages, prevextensions=tuple(),
-                          extincludedirs=None):
+def get_cython_extensions(
+    srcdir, packages, prevextensions=tuple(), extincludedirs=None
+):
     """
     Looks for Cython files and generates Extensions if needed.
 
@@ -513,19 +555,20 @@ def get_cython_extensions(srcdir, packages, prevextensions=tuple(),
 
     for ext in prevextensions:
         for s in ext.sources:
-            if s.endswith(('.pyx', '.c', '.cpp')):
+            if s.endswith((".pyx", ".c", ".cpp")):
                 sourcepath = os.path.realpath(os.path.splitext(s)[0])
                 prevsourcepaths.append(sourcepath)
 
     for package_name in packages:
-        package_parts = package_name.split('.')
+        package_parts = package_name.split(".")
         package_path = os.path.join(srcdir, *package_parts)
 
         for extmod, pyxfn in iter_pyx_files(package_path, package_name):
             sourcepath = os.path.realpath(os.path.splitext(pyxfn)[0])
             if sourcepath not in prevsourcepaths:
-                ext_modules.append(Extension(extmod, [pyxfn],
-                                             include_dirs=extincludedirs))
+                ext_modules.append(
+                    Extension(extmod, [pyxfn], include_dirs=extincludedirs)
+                )
 
     return ext_modules
 
@@ -538,19 +581,19 @@ class DistutilsExtensionArgs(collections.defaultdict):
     `distutils.Extension` without worrying whether the entry is
     already present.
     """
+
     def __init__(self, *args, **kwargs):
         def default_factory():
             return []
 
-        super(DistutilsExtensionArgs, self).__init__(
-            default_factory, *args, **kwargs)
+        super(DistutilsExtensionArgs, self).__init__(default_factory, *args, **kwargs)
 
     def update(self, other):
         for key, val in other.items():
             self[key].extend(val)
 
 
-def pkg_config(packages, default_libraries, executable='pkg-config'):
+def pkg_config(packages, default_libraries, executable="pkg-config"):
     """
     Uses pkg-config to update a set of distutils Extension arguments
     to include the flags necessary to link against the given packages.
@@ -581,9 +624,14 @@ def pkg_config(packages, default_libraries, executable='pkg-config'):
           the compiler
     """
 
-    flag_map = {'-I': 'include_dirs', '-L': 'library_dirs', '-l': 'libraries',
-                '-D': 'define_macros', '-U': 'undef_macros'}
-    command = "{0} --libs --cflags {1}".format(executable, ' '.join(packages)),
+    flag_map = {
+        "-I": "include_dirs",
+        "-L": "library_dirs",
+        "-l": "libraries",
+        "-D": "define_macros",
+        "-U": "undef_macros",
+    }
+    command = ("{0} --libs --cflags {1}".format(executable, " ".join(packages)),)
 
     result = DistutilsExtensionArgs()
 
@@ -592,23 +640,23 @@ def pkg_config(packages, default_libraries, executable='pkg-config'):
         output = pipe.communicate()[0].strip()
     except subprocess.CalledProcessError as e:
         lines = [
-            ("{0} failed. This may cause the build to fail below."
-             .format(executable)),
+            ("{0} failed. This may cause the build to fail below.".format(executable)),
             "  command: {0}".format(e.cmd),
             "  returncode: {0}".format(e.returncode),
-            "  output: {0}".format(e.output)
-            ]
-        log.warn('\n'.join(lines))
-        result['libraries'].extend(default_libraries)
+            "  output: {0}".format(e.output),
+        ]
+        log.warn("\n".join(lines))
+        result["libraries"].extend(default_libraries)
     else:
         if pipe.returncode != 0:
             lines = [
                 "pkg-config could not lookup up package(s) {0}.".format(
-                    ", ".join(packages)),
-                "This may cause the build to fail below."
-                ]
-            log.warn('\n'.join(lines))
-            result['libraries'].extend(default_libraries)
+                    ", ".join(packages)
+                ),
+                "This may cause the build to fail below.",
+            ]
+            log.warn("\n".join(lines))
+            result["libraries"].extend(default_libraries)
         else:
             for token in output.split():
                 # It's not clear what encoding the output of
@@ -618,14 +666,14 @@ def pkg_config(packages, default_libraries, executable='pkg-config'):
                 # that includes directories or filenames), but this is
                 # just conjecture, as the pkg-config documentation
                 # doesn't seem to address it.
-                arg = token[:2].decode('ascii')
+                arg = token[:2].decode("ascii")
                 value = token[2:].decode(sys.getfilesystemencoding())
                 if arg in flag_map:
-                    if arg == '-D':
-                        value = tuple(value.split('=', 1))
+                    if arg == "-D":
+                        value = tuple(value.split("=", 1))
                     result[flag_map[arg]].append(value)
                 else:
-                    result['extra_compile_args'].append(value)
+                    result["extra_compile_args"].append(value)
 
     return result
 
@@ -641,10 +689,13 @@ def add_external_library(library):
         option will be called `--use-system-foo`.
     """
 
-    for command in ['build', 'build_ext', 'install']:
-        add_command_option(command, str('use-system-' + library),
-                           'Use the system {0} library'.format(library),
-                           is_bool=True)
+    for command in ["build", "build_ext", "install"]:
+        add_command_option(
+            command,
+            str("use-system-" + library),
+            "Use the system {0} library".format(library),
+            is_bool=True,
+        )
 
 
 def use_system_library(library):
@@ -667,24 +718,37 @@ def use_system_library(library):
     use_system : bool
         `True` if the build should use the system copy of the library.
     """
-    return (
-        get_distutils_build_or_install_option('use_system_{0}'.format(library)) or
-        get_distutils_build_or_install_option('use_system_libraries'))
+    return get_distutils_build_or_install_option(
+        "use_system_{0}".format(library)
+    ) or get_distutils_build_or_install_option("use_system_libraries")
 
 
 @extends_doc(_find_packages)
-def find_packages(where='.', exclude=(), invalidate_cache=False):
+def find_packages(where=".", exclude=(), invalidate_cache=False):
     """
     This version of ``find_packages`` caches previous results to speed up
     subsequent calls.  Use ``invalide_cache=True`` to ignore cached results
     from previous ``find_packages`` calls, and repeat the package search.
     """
 
-    if not invalidate_cache and _module_state['package_cache'] is not None:
-        return _module_state['package_cache']
+    if exclude:
+        warnings.warn(
+            "Use of the exclude parameter is no longer supported since it does "
+            "not work as expected. Use add_exclude_packages instead. Note that "
+            "it must be called prior to any other calls from setup helpers.",
+            AstropyDeprecationWarning,
+        )
 
-    packages = _find_packages(where=where, exclude=exclude)
-    _module_state['package_cache'] = packages
+    # Calling add_exclude_packages after this point will have no effect
+    _module_state["excludes_too_late"] = True
+
+    if not invalidate_cache and _module_state["package_cache"] is not None:
+        return _module_state["package_cache"]
+
+    packages = _find_packages(
+        where=where, exclude=list(_module_state["exclude_packages"])
+    )
+    _module_state["package_cache"] = packages
 
     return packages
 
@@ -696,9 +760,9 @@ def filter_packages(packagenames):
     """
 
     if PY3:
-        exclude = '_py2'
+        exclude = "_py2"
     else:
-        exclude = '_py3'
+        exclude = "_py3"
 
     return [x for x in packagenames if not x.endswith(exclude)]
 
@@ -711,29 +775,33 @@ class FakeBuildSphinx(Command):
 
     # user options inherited from sphinx.setup_command.BuildDoc
     user_options = [
-        ('fresh-env', 'E', ''),
-        ('all-files', 'a', ''),
-        ('source-dir=', 's', ''),
-        ('build-dir=', None, ''),
-        ('config-dir=', 'c', ''),
-        ('builder=', 'b', ''),
-        ('project=', None, ''),
-        ('version=', None, ''),
-        ('release=', None, ''),
-        ('today=', None, ''),
-        ('link-index', 'i', '')]
+        ("fresh-env", "E", ""),
+        ("all-files", "a", ""),
+        ("source-dir=", "s", ""),
+        ("build-dir=", None, ""),
+        ("config-dir=", "c", ""),
+        ("builder=", "b", ""),
+        ("project=", None, ""),
+        ("version=", None, ""),
+        ("release=", None, ""),
+        ("today=", None, ""),
+        ("link-index", "i", ""),
+    ]
 
     # user options appended in astropy.setup_helpers.AstropyBuildSphinx
-    user_options.append(('warnings-returncode', 'w', ''))
-    user_options.append(('clean-docs', 'l', ''))
-    user_options.append(('no-intersphinx', 'n', ''))
-    user_options.append(('open-docs-in-browser', 'o', ''))
+    user_options.append(("warnings-returncode", "w", ""))
+    user_options.append(("clean-docs", "l", ""))
+    user_options.append(("no-intersphinx", "n", ""))
+    user_options.append(("open-docs-in-browser", "o", ""))
 
     def initialize_options(self):
         try:
-            raise RuntimeError("Sphinx and its dependencies must be installed "
-                               "for build_docs.")
+            raise RuntimeError(
+                "Sphinx and its dependencies must be installed " "for build_docs."
+            )
         except:
-            log.error('error: Sphinx and its dependencies must be installed '
-                      'for build_docs.')
+            log.error(
+                "error: Sphinx and its dependencies must be installed "
+                "for build_docs."
+            )
             sys.exit(1)
